@@ -32,7 +32,12 @@ using is_supported_integral
     = disjunction<std::is_same<T, bool>, std::is_same<T, char>, std::is_same<T, signed char>,
                   std::is_same<T, unsigned char>, std::is_same<T, short>, std::is_same<T, unsigned short>,
                   std::is_same<T, int>, std::is_same<T, unsigned>, std::is_same<T, long>,
-                  std::is_same<T, unsigned long>, std::is_same<T, long long>, std::is_same<T, unsigned long long>>;
+                  std::is_same<T, unsigned long>, std::is_same<T, long long>, std::is_same<T, unsigned long long>
+#if defined(MPPP_HAVE_GCC_INT128)
+                  ,
+                  std::is_same<T, __int128_t>, std::is_same<T, __uint128_t>
+#endif
+                  >;
 
 // Type trait to check if T is a supported floating-point type.
 template <typename T>
@@ -73,6 +78,13 @@ concept bool StringType = is_string_type<T>::value;
 #else
 using string_type_enabler = enable_if_t<is_string_type<T>::value, int>;
 #endif
+
+template <typename T>
+#if defined(MPPP_HAVE_CONCEPTS)
+concept bool CppIntegralInteroperable = is_supported_integral<T>::value;
+#else
+using cpp_integral_interoperable_enabler = enable_if_t<is_supported_integral<T>::value, int>;
+#endif
 }
 
 #endif
