@@ -11,9 +11,9 @@
 
 // Start of defines instantiated by CMake.
 // clang-format off
-#define MPPP_VERSION 0.5
+#define MPPP_VERSION_STRING "0.6"
 #define MPPP_VERSION_MAJOR 0
-#define MPPP_VERSION_MINOR 5
+#define MPPP_VERSION_MINOR 6
 #define MPPP_WITH_MPFR
 
 // clang-format on
@@ -96,4 +96,15 @@
 
 #endif
 
+// Detect if the compiler supports GCC-style 128-bit integers.
+// NOTE: we can check int128 on GCC/clang with __SIZEOF_INT128__ apparently:
+// http://stackoverflow.com/questions/21886985/what-gcc-versions-support-the-int128-intrinsic-type
+// NOTE: clang-cl supports the types, but not all the arithmetic operations on them. Let's disable it for now.
+#if ((defined(__clang__) && !defined(_MSC_VER)) || defined(__GNUC__) || defined(__INTEL_COMPILER))                     \
+    && defined(__SIZEOF_INT128__)
+
+#define MPPP_HAVE_GCC_INT128
+
+#endif
+
 #endif
