@@ -10,8 +10,15 @@
 #define MPPP_MPPP_HPP
 
 #include <mp++/config.hpp>
+
 #include <mp++/exceptions.hpp>
 #include <mp++/integer.hpp>
 #include <mp++/rational.hpp>
+#if defined(MPPP_WITH_MPFR)
+#include <mp++/real.hpp>
+#endif
+#if defined(MPPP_WITH_QUADMATH)
+#include <mp++/real128.hpp>
+#endif
 
 #endif
