@@ -123,12 +123,12 @@ mpq_struct_t get_mpq_view(const rational<SSize> &);
  *    rational<1> q = 5;
  *    int m = q;
  *
- * will not work, and direct initialization and explicit casting should be used instead:
+ * will not work, and direct initialization should be used instead:
  *
  * .. code-block:: c++
  *
  *    rational<1> q{5};
- *    int m = static_cast<int>(q);
+ *    int m{q};
  *
  * Most of the functionality is exposed via plain :ref:`functions <rational_functions>`, with the
  * general convention that the functions are named after the corresponding GMP functions minus the leading ``mpq_``
@@ -244,11 +244,11 @@ public:
         assert(other.m_den.is_static());
         // Set other's numerator to 0 (see integer::set_zero()).
         other.m_num.m_int.g_st()._mp_size = 0;
-        std::fill(other.m_num.m_int.g_st().m_limbs.begin(), other.m_num.m_int.g_st().m_limbs.end(), ::mp_limb_t(0));
+        other.m_num.m_int.g_st().zero_upper_limbs(0);
         // Set other's denominator to 1 (see integer::set_one()).
         other.m_den.m_int.g_st()._mp_size = 1;
         other.m_den.m_int.g_st().m_limbs[0] = 1;
-        std::fill(other.m_den.m_int.g_st().m_limbs.begin() + 1, other.m_den.m_int.g_st().m_limbs.end(), ::mp_limb_t(0));
+        other.m_den.m_int.g_st().zero_upper_limbs(1);
     }
 
 private:
@@ -260,7 +260,7 @@ private:
     {
         if (mppp_unlikely(!std::isfinite(x))) {
             throw std::domain_error("Cannot construct a rational from the non-finite floating-point value "
-                                    + std::to_string(x));
+                                    + mppp::to_string(x));
         }
         MPPP_MAYBE_TLS mpq_raii q;
         ::mpq_set_d(&q.m_mpq, static_cast<double>(x));
@@ -272,7 +272,7 @@ private:
     {
         if (mppp_unlikely(!std::isfinite(x))) {
             throw std::domain_error("Cannot construct a rational from the non-finite floating-point value "
-                                    + std::to_string(x));
+                                    + mppp::to_string(x));
         }
         // NOTE: static checks for overflows and for the precision value are done in mpfr.hpp.
         constexpr int d2 = std::numeric_limits<long double>::max_digits10 * 4;
@@ -281,6 +281,8 @@ private:
         MPPP_MAYBE_TLS mpq_raii mpq;
         // NOTE: we go through an mpfr->mpf->mpq conversion chain as
         // mpfr_get_q() does not exist.
+        // NOTE: probably coming in MPFR 4:
+        // https://lists.gforge.inria.fr/pipermail/mpfr-commits/2017-June/011186.html
         ::mpfr_set_ld(&mpfr.m_mpfr, x, MPFR_RNDN);
         ::mpfr_get_f(&mpf.m_mpf, &mpfr.m_mpfr, MPFR_RNDN);
         ::mpq_set_f(&mpq.m_mpq, &mpf.m_mpf);
@@ -294,17 +296,17 @@ private:
     }
 
 public:
-/// Generic constructor.
-/**
- * \rststar
- * This constructor will initialize a rational with the value ``x``. The construction will fail if ``x``
- * is a non-finite floating-point value.
- * \endrststar
- *
- * @param x the value that will be used to initialize \p this.
- *
- * @throws std::domain_error if \p x is a non-finite floating-point value.
- */
+    /// Generic constructor.
+    /**
+     * \rststar
+     * This constructor will initialize a rational with the value ``x``. The construction will fail if ``x``
+     * is a non-finite floating-point value.
+     * \endrststar
+     *
+     * @param x the value that will be used to initialize \p this.
+     *
+     * @throws std::domain_error if \p x is a non-finite floating-point value.
+     */
 #if defined(MPPP_HAVE_CONCEPTS)
     explicit rational(RationalCvrInteroperable<SSize> &&x)
 #else
@@ -720,8 +722,7 @@ private:
         return std::make_pair(true, m_num.m_int.m_st._mp_size != 0);
     }
     // Conversion to integral types other than bool.
-    template <typename T,
-              enable_if_t<conjunction<std::is_integral<T>, negation<std::is_same<bool, T>>>::value, int> = 0>
+    template <typename T, enable_if_t<conjunction<is_integral<T>, negation<std::is_same<bool, T>>>::value, int> = 0>
     std::pair<bool, T> dispatch_conversion() const
     {
         return static_cast<int_t>(*this).template dispatch_conversion<T>();
@@ -772,7 +773,7 @@ public:
     {
         auto retval = dispatch_conversion<T>();
         if (mppp_unlikely(!retval.first)) {
-            throw std::overflow_error("Conversion of the rational " + to_string() + " to the type " + typeid(T).name()
+            throw std::overflow_error("Conversion of the rational " + to_string() + " to the type " + type_string<T>()
                                       + " results in overflow");
         }
         return std::move(retval.second);
@@ -1809,6 +1810,37 @@ inline T &operator+=(T &rop, const U &op)
     return rop;
 }
 
+/// Prefix increment for \link mppp::rational rational\endlink.
+/**
+ * This operator will increment \p q by one.
+ *
+ * @param q the \link mppp::rational rational\endlink that will be increased.
+ *
+ * @return a reference to \p q after the increment.
+ */
+template <std::size_t SSize>
+inline rational<SSize> &operator++(rational<SSize> &q)
+{
+    add(q._get_num(), q._get_num(), q.get_den());
+    return q;
+}
+
+/// Suffix increment for \link mppp::rational rational\endlink.
+/**
+ * This operator will increment \p q by one and return a copy of \p q as it was before the increment.
+ *
+ * @param q the \link mppp::rational rational\endlink that will be increased.
+ *
+ * @return a copy of \p q before the increment.
+ */
+template <std::size_t SSize>
+inline rational<SSize> operator++(rational<SSize> &q, int)
+{
+    auto retval(q);
+    ++q;
+    return retval;
+}
+
 /// Negated copy.
 /**
  * @param q the rational that will be negated.
@@ -1966,6 +1998,37 @@ inline T &operator-=(T &rop, const U &op)
     return rop;
 }
 
+/// Prefix decrement for \link mppp::rational rational\endlink.
+/**
+ * This operator will decrement \p q by one.
+ *
+ * @param q the \link mppp::rational rational\endlink that will be decreased.
+ *
+ * @return a reference to \p q after the decrement.
+ */
+template <std::size_t SSize>
+inline rational<SSize> &operator--(rational<SSize> &q)
+{
+    sub(q._get_num(), q._get_num(), q.get_den());
+    return q;
+}
+
+/// Suffix decrement for \link mppp::rational rational\endlink.
+/**
+ * This operator will decrement \p q by one and return a copy of \p q as it was before the decrement.
+ *
+ * @param q the \link mppp::rational rational\endlink that will be decreased.
+ *
+ * @return a copy of \p q before the decrement.
+ */
+template <std::size_t SSize>
+inline rational<SSize> operator--(rational<SSize> &q, int)
+{
+    auto retval(q);
+    --q;
+    return retval;
+}
+
 inline namespace detail
 {
 
