@@ -148,7 +148,7 @@ using real128_op_types_enabler
  * This class represents real values encoded in the quadruple-precision IEEE 754 floating-point format
  * (which features up to 36 decimal digits of precision).
  * The class is a thin wrapper around the :cpp:type:`__float128` type and the quadmath library, available in GCC
- * and recent Clang versions on most contemporary platforms, on top of which it provides the following additions:
+ * and recent Clang versions on most modern platforms, on top of which it provides the following additions:
  *
  * * interoperability with other mp++ classes,
  * * consistent behaviour with respect to the conventions followed elsewhere in mp++ (e.g., values are
@@ -168,12 +168,12 @@ using real128_op_types_enabler
  *    real128 r = 5.23;
  *    int m = r;
  *
- * will not work, and direct initialization and explicit casting should be used instead:
+ * will not work, and direct initialization should be used instead:
  *
  * .. code-block:: c++
  *
  *    real128 r{5.23};
- *    int m = static_cast<int>(r);
+ *    int m{r};
  *
  * Most of the functionality is exposed via plain :ref:`functions <real128_functions>`, with the
  * general convention that the functions are named after the corresponding quadmath functions minus the trailing ``q``
@@ -728,6 +728,36 @@ public:
                                     + (is_integer<T>::value ? "an integer" : "a rational"));
         }
         return retval;
+    }
+        /// Conversion method to interoperable C++ types.
+        /**
+         * \rststar
+         * This method will cast the :cpp:member:`~mppp::real128::m_value` member to the
+         * :cpp:concept:`~mppp::Real128CppInteroperable` type T, and assign the result
+         * to ``rop``.
+         *
+         * .. note::
+         *    This method is provided for consistency with other getter methods,
+         *    and it does not offer any additional functionality over the plain
+         *    conversion operator.
+         *
+         * .. note::
+         *
+         *   This method is marked as ``constexpr`` only if at least C++14 is being used.
+         * \endrststar
+         *
+         * @param rop the variable which will store the result of the conversion.
+         *
+         * @return always ``true``.
+         */
+#if defined(MPPP_HAVE_CONCEPTS)
+    template <Real128CppInteroperable T>
+#else
+    template <typename T, real128_cpp_interoperable_enabler<T> = 0>
+#endif
+    MPPP_CONSTEXPR_14 bool get(T &rop) const
+    {
+        return rop = static_cast<T>(m_value), true;
     }
         /// Conversion method to mp++ types.
         /**
@@ -980,6 +1010,38 @@ public:
  *  @{
  */
 
+/// Conversion function to C++ types for \link mppp::real128 real128\endlink.
+/**
+ * \rststar
+ * This function will convert the input :cpp:class:`~mppp::real128` ``x`` to a
+ * :cpp:concept:`~mppp::Real128CppInteroperable` type, storing the result of the conversion into ``rop``.
+ * The conversion is always successful.
+ *
+ * .. note::
+ *    This function is provided for consistency with other getter functions,
+ *    and it does not offer any additional functionality over the plain
+ *    conversion operator.
+ *
+ * .. note::
+ *
+ *   This function is marked as ``constexpr`` only if at least C++14 is being used.
+ * \endrststar
+ *
+ * @param rop the variable which will store the result of the conversion.
+ * @param x the input \link mppp::real128 real128\endlink.
+ *
+ * @return always ``true``.
+ */
+#if defined(MPPP_HAVE_CONCEPTS)
+inline MPPP_CONSTEXPR_14 bool get(Real128CppInteroperable &rop, const real128 &x)
+#else
+template <typename T, real128_cpp_interoperable_enabler<T> = 0>
+inline MPPP_CONSTEXPR_14 bool get(T &rop, const real128 &x)
+#endif
+{
+    return x.get(rop);
+}
+
 /// Conversion function to mp++ types for \link mppp::real128 real128\endlink.
 /**
  * \rststar
@@ -1006,6 +1068,26 @@ inline bool get(T &rop, const real128 &x)
     return x.get(rop);
 }
 
+/// Decompose a \link mppp::real128 real128\endlink into a normalized fraction and an integral power of two.
+/**
+ * \rststar
+ * If ``x`` is zero, this function will return zero and store zero in ``exp``. Otherwise,
+ * this function will return a :cpp:class:`~mppp::real128` :math:`r` with an absolute value in the
+ * :math:`\left[0.5,1\right)` range, and it will store an integer value :math:`n` in ``exp``
+ * such that :math:`r \times 2^n` equals to :math:`r`. If ``x`` is a non-finite value, the return
+ * value will be ``x`` and an unspecified value will be stored in ``exp``.
+ * \endrststar
+ *
+ * @param x the input \link mppp::real128 real128\endlink.
+ * @param exp a pointer to the value that will store the exponent.
+ *
+ * @return the binary significand of ``x``.
+ */
+inline real128 frexp(const real128 &x, int *exp)
+{
+    return real128{::frexpq(x.m_value, exp)};
+}
+
 /** @} */
 
 /** @defgroup real128_arithmetic real128_arithmetic
