@@ -1,5 +1,11 @@
 #!/usr/bin/env bash
 
+if [[ "$(uname)" == "Darwin" ]]; then
+    export ENABLE_QUADMATH=no
+else
+    export ENABLE_QUADMATH=yes
+fi
+
 mkdir build
 cd build
 
@@ -8,6 +14,7 @@ cmake \
     -DCMAKE_INSTALL_PREFIX=$PREFIX \
     -DCMAKE_PREFIX_PATH=$PREFIX \
     -DMPPP_WITH_MPFR=yes \
+    -DMPPP_WITH_QUADMATH=$ENABLE_QUADMATH \
     -DMPPP_BUILD_TESTS=yes \
     ..
 
